/*
 * Definitions for Freescale LITE5200 : MPC52xx Standard Development
 * Platform board support
 * 
 * Maintainer : Sylvain Munaut <tnt@246tNt.com>
 * 
 * Copyright (C) 2004 Sylvain Munaut <tnt@246tNt.com>
 *
 * This file is licensed under the terms of the GNU General Public License
 * version 2. This program is licensed "as is" without any warranty of any
 * kind, whether express or implied.
 */

#ifndef __PLATFORMS_LITE5200_H__
#define __PLATFORMS_LITE5200_H__

/* Serial port used for low-level debug */
#define MPC52xx_PF_CONSOLE_PORT 1	/* PSC1 */


#endif /* __PLATFORMS_LITE5200_H__ */
