void libertas_debugfs_init(void);
void libertas_debugfs_remove(void);

void libertas_debugfs_init_one(wlan_private *priv, struct net_device *dev);
void libertas_debugfs_remove_one(wlan_private *priv);

