/*
 *  linux/include/asm-arm/arch-ebsa110/vmalloc.h
 *
 *  Copyright (C) 1998 Russell King
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
#define VMALLOC_END       (PAGE_OFFSET + 0x1f000000)
