/*
 * linux/sound/soc/pxa/pxa2xx-i2s.h
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifndef _PXA2XX_I2S_H
#define _PXA2XX_I2S_H

/* pxa2xx DAI ID's */
#define PXA2XX_DAI_I2S			0

/* I2S clock */
#define PXA2XX_I2S_SYSCLK		0

extern struct snd_soc_cpu_dai pxa_i2s_dai;

#endif
