/*
 * include/asm-arm/arch-ns9xxx/board.h
 *
 * Copyright (C) 2006 by Digi International Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 */
#ifndef __ASM_ARCH_BOARD_H
#define __ASM_ARCH_BOARD_H

#include <asm/mach-types.h>

#define board_is_a9m9750dev()	(machine_is_cc9p9360dev())

#define board_is_jscc9p9360()	(machine_is_cc9p9360js())

#endif /* ifndef __ASM_ARCH_BOARD_H */
