/*
 * include/asm-arm/arch-ixp4xx/dma.h
 *
 * Copyright (C) 2001-2004 MontaVista Software, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */
#ifndef __ASM_ARCH_DMA_H
#define __ASM_ARCH_DMA_H

#include <linux/device.h>
#include <asm/page.h>
#include <asm/sizes.h>
#include <asm/hardware.h>

#define MAX_DMA_ADDRESS		(PAGE_OFFSET + SZ_64M)

#endif /* _ASM_ARCH_DMA_H */
