/*
 *
 * BRIEF MODULE DESCRIPTION
 *  USB specific definitions
 *
 * Author: source@mvista.com
 *
 *  This program is free software; you can distribute it and/or modify it
 *  under the terms of the GNU General Public License (Version 2) as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 */

#ifndef __PNX8550_USB_H
#define __PNX8550_USB_H

/*
 * USB Host controller
 */

#define PNX8550_USB_OHCI_OP_BASE	0x1be48000
#define PNX8550_USB_OHCI_OP_LEN	        0x1000

#endif
