#ifndef _ASM_POWERPC_BUGS_H
#define _ASM_POWERPC_BUGS_H

/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */

/*
 * This file is included by 'init/main.c' to check for
 * architecture-dependent bugs.
 */

static inline void check_bugs(void) { }

#endif	/* _ASM_POWERPC_BUGS_H */
