#ifndef __ASM_AVR32_CACHECTL_H
#define __ASM_AVR32_CACHECTL_H

/*
 * Operations that can be performed through the cacheflush system call
 */

/* Clean the data cache, then invalidate the icache */
#define CACHE_IFLUSH	0

#endif /* __ASM_AVR32_CACHECTL_H */
