#ifndef _ASM_M32R_SYSCALL_H
#define _ASM_M32R_SYSCALL_H

/* Definitions for the system call vector.  */
#define SYSCALL_VECTOR          "2"
#define SYSCALL_VECTOR_ADDRESS  "0xa0"

#endif /* _ASM_M32R_SYSCALL_H */
