#ifndef __ASM_SH64_NAMEI_H
#define __ASM_SH64_NAMEI_H

/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * include/asm-sh64/namei.h
 *
 * Copyright (C) 2000, 2001  Paolo Alberelli
 *
 * Included from linux/fs/namei.c
 *
 */

/* This dummy routine maybe changed to something useful
 * for /usr/gnemul/ emulation stuff.
 * Look at asm-sparc/namei.h for details.
 */

#define __emul_prefix() NULL

#endif /* __ASM_SH64_NAMEI_H */
