#ifndef __ASM_AVR32_DMA_H
#define __ASM_AVR32_DMA_H

/* The maximum address that we can perform a DMA transfer to on this platform.
 * Not really applicable to AVR32, but some functions need it. */
#define MAX_DMA_ADDRESS		0xffffffff

#endif /* __ASM_AVR32_DMA_H */
