/*
 * linux/sound/codecs/ac97.h -- ALSA SoC Layer
 *
 * Author:		Liam Girdwood
 * Created:		Dec 1st 2005
 * Copyright:	Wolfson Microelectronics. PLC.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifndef __LINUX_SND_SOC_AC97_H
#define __LINUX_SND_SOC_AC97_H

extern struct snd_soc_codec_device soc_codec_dev_ac97;
extern struct snd_soc_codec_dai ac97_dai;

#endif
